#include <cstdio>
#include <cstdlib>
#include <cctype>
#include <cstring>

#include "list.h"

const unsigned int MaxBuffer = 1024;
char buffer[MaxBuffer];

const char * CmdFin = "FIN";
const char * CmdAyuda = "?";
const char * CmdCrear = "C";
const char * CmdEliminar = "E";
const char * CmdVer = "V";
const char * CmdVerAtras = "A";
const char * CmdBorrar = "B";


const char * TxtAyuda = "? - Esta ayuda.\n"
                        "c Crear elemento\n"
                        "e Eliminar elemento\n"
                        "v Ver toda la lista\n"
                        "a ver toda la lista hacia atrs\n"
                        "b Borrar todo el lista\n"
;

const char * MsgErrorInesperado = "Error inesperado";
const char * MsgPosicionIncorrecta = "Posicin incorrecta";
const char * MsgRangoIncorrecto = "Rango incorrecto";
const char * MsgSinMemoria = "Sin memoria";
const char * MsgOk = "Ok";

char * maysCnvt(char * buf)
{
    char * ptr = buf;

    for(; *ptr != 0; ++ptr) {
        *ptr = toupper( *ptr );
    }

    return buf;
}

char * trimCnvt(char * buf)
{
    static const char * Espacios = " \t\n\r";
    char * fin = buf + strlen( buf );
    char * inicioReal;
    char * finReal;

    // Encontrar el inicio real
    for(inicioReal = buf; strchr( Espacios, *inicioReal ) != NULL; ++inicioReal);

    // Encontrar el fin real
    for(finReal = fin - 1; strchr( Espacios, *finReal ) != NULL; --finReal);

    // Mover el texto
    *( finReal + 1 ) = 0;
    strcpy( buf, inicioReal );

    return buf;
}

void eliminarLista(List l);

const char * interpretarError(ListStatus st)
{
    const char * toret = MsgErrorInesperado;

    switch( st ) {
        case ListInvalidRange:
            toret = MsgRangoIncorrecto;
            break;
        case ListInvalidPos:
            toret = MsgPosicionIncorrecta;
            break;
        case ListOk:
            toret = MsgOk;
            break;
        case ListNoMemory:
            toret = MsgSinMemoria;
            break;

    }

    return toret;
}

void crearLista(List &l)
{
    if ( l != NULL ) {
        destroyList( l );
    }

    l = createList();
    printf( "Crear la lista" );
}

void crearElemento(List l)
{
    if ( l != NULL ) {
        printf( "Introduzca una cadena: " );
        gets( buffer );

        char * cad = strdup( buffer );

        if ( cad != NULL )
                insertListElement( l, getListFirstPos( l ), cad );
        else    printf( "Sin memoria\n" );
    }
    else printf( "Lista inexistente." );
}

void limpiarPos(List l)
{
    int pos;
    Position p = getListFirstPos( l );

    if ( l != NULL ) {
        printf( "Introduzca la posicin a eliminar: " );
        scanf( "%d", &pos );
        advanceListPos( l, p, pos );

        deleteListElement( l, p );
    }
    else printf( "Lista inexistente." );
}

int main()
{
    List l = createList();
    bool orden;

    *buffer = 0;
    do {
        orden = true;
        printf( "Introduzca orden (? = ayuda): " );
        fflush( stdin );
        gets( buffer );

        maysCnvt( trimCnvt( buffer ) );

        if ( !strcmp( buffer, CmdAyuda ) ) {
            printf( "\n%s\n", TxtAyuda );
        }
        else
        if ( !strcmp( buffer, CmdCrear ) ) {
            crearElemento( l );
        }
        else
        if ( !strcmp( buffer, CmdEliminar ) ) {
            limpiarPos( l );
        }
        else
        if ( !strcmp( buffer, CmdBorrar ) ) {
             deleteListRange( l, getListFirstPos( l ), getListLastPos( l ) );
        }
        else
        if ( !strcmp( buffer, CmdVerAtras ) ) {
            if ( l != NULL ) {
                Position pos = getListLastPos( l );
                printf( "Lista:\n" );

                for(; pos != NULL; pos = getListPreviousPos( l, pos ) ) {
                    printf( "%s\n", (char *) getListPosContent( l, pos ) );
                }

                printf( "\n\n" );
            }
            else printf( "Lista indefinida\n" );
        }
        else
        if ( !strcmp( buffer, CmdVer ) ) {
            if ( l != NULL ) {
                Position pos = getListFirstPos( l );
                printf( "Lista:\n" );

                for(; pos != NULL; pos = getListNextPos( l, pos ) ) {
                    printf( "%s\n", (char *) getListPosContent( l, pos ) );
                }

                printf( "\n\n" );
            }
            else printf( "Lista indefinida\n" );
        }
        else
        if ( !strcmp( buffer, CmdFin ) ) {
            printf( "Saliendo ...\n" );

        }
        else orden = false;

        if ( orden )
                printf( "\n%s\n", interpretarError( getListStatus() ) );
        else    printf( "%s\n", "Comando incorrecto" );

    } while( strcmp( buffer, CmdFin ) );

    eliminarLista( l );
    return 0;
}

void eliminarLista(List l)
{
    Position pos = getListFirstPos( l );

    for(; pos != NULL; pos = getListNextPos( l, pos ) ) {
        free( getListPosContent( l, pos ) );
    }

    destroyList( l );
}
